@@ -8,12 +8,13 @@ from django.contrib.admin.decorators import action
 from django.contrib.admin.utils import model_ngettext
 from django.core.exceptions import PermissionDenied
 from django.template.response import TemplateResponse
-from django.utils.translation import gettext as _, gettext_lazy
+from django.utils.translation import gettext as _
+from django.utils.translation import gettext_lazy
 
 
 @action(
-    permissions=['delete'],
-    description=gettext_lazy('Delete selected %(verbose_name_plural)s'),
+    permissions=["delete"],
+    description=gettext_lazy("Delete selected %(verbose_name_plural)s"),
 )
 def delete_selected(modeladmin, request, queryset):
     """
@@ -30,11 +31,16 @@ def delete_selected(modeladmin, request, queryset):
 
     # Populate deletable_objects, a data structure of all related objects that
     # will also be deleted.
-    deletable_objects, model_count, perms_needed, protected = modeladmin.get_deleted_objects(queryset, request)
+    (
+        deletable_objects,
+        model_count,
+        perms_needed,
+        protected,
+    ) = modeladmin.get_deleted_objects(queryset, request)
 
     # The user has already confirmed the deletion.
     # Do the deletion and return None to display the change list view again.
-    if request.POST.get('post') and not protected:
+    if request.POST.get("post") and not protected:
         if perms_needed:
             raise PermissionDenied
         n = queryset.count()
@@ -43,9 +49,12 @@ def delete_selected(modeladmin, request, queryset):
                 obj_display = str(obj)
                 modeladmin.log_deletion(request, obj, obj_display)
             modeladmin.delete_queryset(request, queryset)
-            modeladmin.message_user(request, _("Successfully deleted %(count)d %(items)s.") % {
-                "count": n, "items": model_ngettext(modeladmin.opts, n)
-            }, messages.SUCCESS)
+            modeladmin.message_user(
+                request,
+                _("Successfully deleted %(count)d %(items)s.")
+                % {"count": n, "items": model_ngettext(modeladmin.opts, n)},
+                messages.SUCCESS,
+            )
         # Return None to display the change list page again.
         return None
 
@@ -58,23 +67,29 @@ def delete_selected(modeladmin, request, queryset):
 
     context = {
         **modeladmin.admin_site.each_context(request),
-        'title': title,
-        'objects_name': str(objects_name),
-        'deletable_objects': [deletable_objects],
-        'model_count': dict(model_count).items(),
-        'queryset': queryset,
-        'perms_lacking': perms_needed,
-        'protected': protected,
-        'opts': opts,
-        'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
-        'media': modeladmin.media,
+        "title": title,
+        "objects_name": str(objects_name),
+        "deletable_objects": [deletable_objects],
+        "model_count": dict(model_count).items(),
+        "queryset": queryset,
+        "perms_lacking": perms_needed,
+        "protected": protected,
+        "opts": opts,
+        "action_checkbox_name": helpers.ACTION_CHECKBOX_NAME,
+        "media": modeladmin.media,
     }
 
     request.current_app = modeladmin.admin_site.name
 
     # Display the confirmation page
-    return TemplateResponse(request, modeladmin.delete_selected_confirmation_template or [
-        "admin/%s/%s/delete_selected_confirmation.html" % (app_label, opts.model_name),
-        "admin/%s/delete_selected_confirmation.html" % app_label,
-        "admin/delete_selected_confirmation.html"
-    ], context)
+    return TemplateResponse(
+        request,
+        modeladmin.delete_selected_confirmation_template
+        or [
+            "admin/%s/%s/delete_selected_confirmation.html"
+            % (app_label, opts.model_name),
+            "admin/%s/delete_selected_confirmation.html" % app_label,
+            "admin/delete_selected_confirmation.html",
+        ],
+        context,
+    )
