@@ -1,14 +1,11 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from django.db import migrations
 
 
 class Migration(migrations.Migration):
 
     dependencies = [
-        ('auth', '0005_alter_user_last_login_null'),
-        ('contenttypes', '0002_remove_content_type_name'),
+        ("auth", "0005_alter_user_last_login_null"),
+        ("contenttypes", "0002_remove_content_type_name"),
     ]
 
     operations = [
