@@ -1,9 +1,14 @@
 import struct
 
-from django.forms import ValidationError
+from django.core.exceptions import ValidationError
 
 from .const import (
-    GDAL_TO_POSTGIS, GDAL_TO_STRUCT, POSTGIS_HEADER_STRUCTURE, POSTGIS_TO_GDAL,
+    BANDTYPE_FLAG_HASNODATA,
+    BANDTYPE_PIXTYPE_MASK,
+    GDAL_TO_POSTGIS,
+    GDAL_TO_STRUCT,
+    POSTGIS_HEADER_STRUCTURE,
+    POSTGIS_TO_GDAL,
     STRUCT_SIZE,
 )
 
@@ -12,14 +17,14 @@ def pack(structure, data):
     """
     Pack data into hex string with little endian format.
     """
-    return struct.pack('<' + structure, *data)
+    return struct.pack("<" + structure, *data)
 
 
 def unpack(structure, data):
     """
     Unpack little endian hexlified binary string into a list.
     """
-    return struct.unpack('<' + structure, bytes.fromhex(data))
+    return struct.unpack("<" + structure, bytes.fromhex(data))
 
 
 def chunk(data, index):
@@ -45,13 +50,9 @@ def from_pgraster(data):
     pixeltypes = []
     while data:
         # Get pixel type for this band
-        pixeltype, data = chunk(data, 2)
-        pixeltype = unpack('B', pixeltype)[0]
-
-        # Subtract nodata byte from band nodata value if it exists
-        has_nodata = pixeltype >= 64
-        if has_nodata:
-            pixeltype -= 64
+        pixeltype_with_flags, data = chunk(data, 2)
+        pixeltype_with_flags = unpack("B", pixeltype_with_flags)[0]
+        pixeltype = pixeltype_with_flags & BANDTYPE_PIXTYPE_MASK
 
         # Convert datatype from PostGIS to GDAL & get pack type and size
         pixeltype = POSTGIS_TO_GDAL[pixeltype]
@@ -66,11 +67,11 @@ def from_pgraster(data):
 
         # Chunk and unpack band data (pack size times nr of pixels)
         band, data = chunk(data, pack_size * header[10] * header[11])
-        band_result = {'data': bytes.fromhex(band)}
+        band_result = {"data": bytes.fromhex(band)}
 
-        # If the nodata flag is True, set the nodata value.
-        if has_nodata:
-            band_result['nodata_value'] = nodata
+        # Set the nodata value if the nodata flag is set.
+        if pixeltype_with_flags & BANDTYPE_FLAG_HASNODATA:
+            band_result["nodata_value"] = nodata
 
         # Append band data to band list
         bands.append(band_result)
@@ -85,13 +86,14 @@ def from_pgraster(data):
         raise ValidationError("Band pixeltypes are not all equal.")
 
     return {
-        'srid': int(header[9]),
-        'width': header[10], 'height': header[11],
-        'datatype': pixeltypes[0],
-        'origin': (header[5], header[6]),
-        'scale': (header[3], header[4]),
-        'skew': (header[7], header[8]),
-        'bands': bands,
+        "srid": int(header[9]),
+        "width": header[10],
+        "height": header[11],
+        "datatype": pixeltypes[0],
+        "origin": (header[5], header[6]),
+        "scale": (header[3], header[4]),
+        "skew": (header[7], header[8]),
+        "bands": bands,
     }
 
 
@@ -103,9 +105,18 @@ def to_pgraster(rast):
     # the endianness and the PostGIS Raster Version, both are fixed by
     # PostGIS at the moment.
     rasterheader = (
-        1, 0, len(rast.bands), rast.scale.x, rast.scale.y,
-        rast.origin.x, rast.origin.y, rast.skew.x, rast.skew.y,
-        rast.srs.srid, rast.width, rast.height,
+        1,
+        0,
+        len(rast.bands),
+        rast.scale.x,
+        rast.scale.y,
+        rast.origin.x,
+        rast.origin.y,
+        rast.skew.x,
+        rast.skew.y,
+        rast.srs.srid,
+        rast.width,
+        rast.height,
     )
 
     # Pack raster header.
@@ -116,20 +127,21 @@ def to_pgraster(rast):
         # and the nodata value.
         #
         # The 8BUI stores both the PostGIS pixel data type and a nodata flag.
-        # It is composed as the datatype integer plus 64 as a flag for existing
-        # nodata values:
-        # 8BUI_VALUE = PG_PIXEL_TYPE (0-11) + FLAG (0 or 64)
+        # It is composed as the datatype with BANDTYPE_FLAG_HASNODATA (1 << 6)
+        # for existing nodata values:
+        #   8BUI_VALUE = PG_PIXEL_TYPE (0-11) | BANDTYPE_FLAG_HASNODATA
         #
         # For example, if the byte value is 71, then the datatype is
-        # 71-64 = 7 (32BSI) and the nodata value is True.
-        structure = 'B' + GDAL_TO_STRUCT[band.datatype()]
+        #   71 & ~BANDTYPE_FLAG_HASNODATA = 7 (32BSI)
+        # and the nodata value is True.
+        structure = "B" + GDAL_TO_STRUCT[band.datatype()]
 
         # Get band pixel type in PostGIS notation
         pixeltype = GDAL_TO_POSTGIS[band.datatype()]
 
         # Set the nodata flag
         if band.nodata_value is not None:
-            pixeltype += 64
+            pixeltype |= BANDTYPE_FLAG_HASNODATA
 
         # Pack band header
         bandheader = pack(structure, (pixeltype, band.nodata_value or 0))
