@@ -14,27 +14,35 @@ See docs/topics/cache.txt for information on the public API.
 """
 from django.core import signals
 from django.core.cache.backends.base import (
-    BaseCache, CacheKeyWarning, InvalidCacheBackendError, InvalidCacheKey,
+    BaseCache,
+    CacheKeyWarning,
+    InvalidCacheBackendError,
+    InvalidCacheKey,
 )
 from django.utils.connection import BaseConnectionHandler, ConnectionProxy
 from django.utils.module_loading import import_string
 
 __all__ = [
-    'cache', 'caches', 'DEFAULT_CACHE_ALIAS', 'InvalidCacheBackendError',
-    'CacheKeyWarning', 'BaseCache', 'InvalidCacheKey',
+    "cache",
+    "caches",
+    "DEFAULT_CACHE_ALIAS",
+    "InvalidCacheBackendError",
+    "CacheKeyWarning",
+    "BaseCache",
+    "InvalidCacheKey",
 ]
 
-DEFAULT_CACHE_ALIAS = 'default'
+DEFAULT_CACHE_ALIAS = "default"
 
 
 class CacheHandler(BaseConnectionHandler):
-    settings_name = 'CACHES'
+    settings_name = "CACHES"
     exception_class = InvalidCacheBackendError
 
     def create_connection(self, alias):
         params = self.settings[alias].copy()
-        backend = params.pop('BACKEND')
-        location = params.pop('LOCATION', '')
+        backend = params.pop("BACKEND")
+        location = params.pop("LOCATION", "")
         try:
             backend_cls = import_string(backend)
         except ImportError as e:
@@ -43,6 +51,14 @@ class CacheHandler(BaseConnectionHandler):
             ) from e
         return backend_cls(location, params)
 
+    def all(self, initialized_only=False):
+        return [
+            self[alias]
+            for alias in self
+            # If initialized_only is True, return only initialized caches.
+            if not initialized_only or hasattr(self._connections, alias)
+        ]
+
 
 caches = CacheHandler()
 
@@ -52,7 +68,7 @@ cache = ConnectionProxy(caches, DEFAULT_CACHE_ALIAS)
 def close_caches(**kwargs):
     # Some caches need to do a cleanup at the end of a request cycle. If not
     # implemented in a particular backend cache.close() is a no-op.
-    for cache in caches.all():
+    for cache in caches.all(initialized_only=True):
         cache.close()
 
 
