@@ -14,7 +14,7 @@ from django.utils.module_loading import import_string
 
 from .exception import convert_exception_to_response
 
-logger = logging.getLogger('django.request')
+logger = logging.getLogger("django.request")
 
 
 class BaseHandler:
@@ -38,12 +38,12 @@ class BaseHandler:
         handler_is_async = is_async
         for middleware_path in reversed(settings.MIDDLEWARE):
             middleware = import_string(middleware_path)
-            middleware_can_sync = getattr(middleware, 'sync_capable', True)
-            middleware_can_async = getattr(middleware, 'async_capable', False)
+            middleware_can_sync = getattr(middleware, "sync_capable", True)
+            middleware_can_async = getattr(middleware, "async_capable", False)
             if not middleware_can_sync and not middleware_can_async:
                 raise RuntimeError(
-                    'Middleware %s must have at least one of '
-                    'sync_capable/async_capable set to True.' % middleware_path
+                    "Middleware %s must have at least one of "
+                    "sync_capable/async_capable set to True." % middleware_path
                 )
             elif not handler_is_async and middleware_can_sync:
                 middleware_is_async = False
@@ -52,35 +52,40 @@ class BaseHandler:
             try:
                 # Adapt handler, if needed.
                 adapted_handler = self.adapt_method_mode(
-                    middleware_is_async, handler, handler_is_async,
-                    debug=settings.DEBUG, name='middleware %s' % middleware_path,
+                    middleware_is_async,
+                    handler,
+                    handler_is_async,
+                    debug=settings.DEBUG,
+                    name="middleware %s" % middleware_path,
                 )
                 mw_instance = middleware(adapted_handler)
             except MiddlewareNotUsed as exc:
                 if settings.DEBUG:
                     if str(exc):
-                        logger.debug('MiddlewareNotUsed(%r): %s', middleware_path, exc)
+                        logger.debug("MiddlewareNotUsed(%r): %s", middleware_path, exc)
                     else:
-                        logger.debug('MiddlewareNotUsed: %r', middleware_path)
+                        logger.debug("MiddlewareNotUsed: %r", middleware_path)
                 continue
             else:
                 handler = adapted_handler
 
             if mw_instance is None:
                 raise ImproperlyConfigured(
-                    'Middleware factory %s returned None.' % middleware_path
+                    "Middleware factory %s returned None." % middleware_path
                 )
 
-            if hasattr(mw_instance, 'process_view'):
+            if hasattr(mw_instance, "process_view"):
                 self._view_middleware.insert(
                     0,
                     self.adapt_method_mode(is_async, mw_instance.process_view),
                 )
-            if hasattr(mw_instance, 'process_template_response'):
+            if hasattr(mw_instance, "process_template_response"):
                 self._template_response_middleware.append(
-                    self.adapt_method_mode(is_async, mw_instance.process_template_response),
+                    self.adapt_method_mode(
+                        is_async, mw_instance.process_template_response
+                    ),
                 )
-            if hasattr(mw_instance, 'process_exception'):
+            if hasattr(mw_instance, "process_exception"):
                 # The exception-handling stack is still always synchronous for
                 # now, so adapt that way.
                 self._exception_middleware.append(
@@ -97,7 +102,12 @@ class BaseHandler:
         self._middleware_chain = handler
 
     def adapt_method_mode(
-        self, is_async, method, method_is_async=None, debug=False, name=None,
+        self,
+        is_async,
+        method,
+        method_is_async=None,
+        debug=False,
+        name=None,
     ):
         """
         Adapt a method to be in the correct "mode":
@@ -111,15 +121,15 @@ class BaseHandler:
         if method_is_async is None:
             method_is_async = asyncio.iscoroutinefunction(method)
         if debug and not name:
-            name = name or 'method %s()' % method.__qualname__
+            name = name or "method %s()" % method.__qualname__
         if is_async:
             if not method_is_async:
                 if debug:
-                    logger.debug('Synchronous %s adapted.', name)
+                    logger.debug("Synchronous %s adapted.", name)
                 return sync_to_async(method, thread_sensitive=True)
         elif method_is_async:
             if debug:
-                logger.debug('Asynchronous %s adapted.' % name)
+                logger.debug("Asynchronous %s adapted.", name)
             return async_to_sync(method)
         return method
 
@@ -131,7 +141,9 @@ class BaseHandler:
         response._resource_closers.append(request.close)
         if response.status_code >= 400:
             log_response(
-                '%s: %s', response.reason_phrase, request.path,
+                "%s: %s",
+                response.reason_phrase,
+                request.path,
                 response=response,
                 request=request,
             )
@@ -151,7 +163,9 @@ class BaseHandler:
         response._resource_closers.append(request.close)
         if response.status_code >= 400:
             await sync_to_async(log_response, thread_sensitive=False)(
-                '%s: %s', response.reason_phrase, request.path,
+                "%s: %s",
+                response.reason_phrase,
+                request.path,
                 response=response,
                 request=request,
             )
@@ -168,7 +182,9 @@ class BaseHandler:
 
         # Apply view middleware
         for middleware_method in self._view_middleware:
-            response = middleware_method(request, callback, callback_args, callback_kwargs)
+            response = middleware_method(
+                request, callback, callback_args, callback_kwargs
+            )
             if response:
                 break
 
@@ -189,16 +205,16 @@ class BaseHandler:
 
         # If the response supports deferred rendering, apply template
         # response middleware and then render the response
-        if hasattr(response, 'render') and callable(response.render):
+        if hasattr(response, "render") and callable(response.render):
             for middleware_method in self._template_response_middleware:
                 response = middleware_method(request, response)
-                # Complain if the template response middleware returned None (a common error).
+                # Complain if the template response middleware returned None
+                # (a common error).
                 self.check_response(
                     response,
                     middleware_method,
-                    name='%s.process_template_response' % (
-                        middleware_method.__self__.__class__.__name__,
-                    )
+                    name="%s.process_template_response"
+                    % (middleware_method.__self__.__class__.__name__,),
                 )
             try:
                 response = response.render()
@@ -220,7 +236,9 @@ class BaseHandler:
 
         # Apply view middleware.
         for middleware_method in self._view_middleware:
-            response = await middleware_method(request, callback, callback_args, callback_kwargs)
+            response = await middleware_method(
+                request, callback, callback_args, callback_kwargs
+            )
             if response:
                 break
 
@@ -228,9 +246,13 @@ class BaseHandler:
             wrapped_callback = self.make_view_atomic(callback)
             # If it is a synchronous view, run it in a subthread
             if not asyncio.iscoroutinefunction(wrapped_callback):
-                wrapped_callback = sync_to_async(wrapped_callback, thread_sensitive=True)
+                wrapped_callback = sync_to_async(
+                    wrapped_callback, thread_sensitive=True
+                )
             try:
-                response = await wrapped_callback(request, *callback_args, **callback_kwargs)
+                response = await wrapped_callback(
+                    request, *callback_args, **callback_kwargs
+                )
             except Exception as e:
                 response = await sync_to_async(
                     self.process_exception_by_middleware,
@@ -244,7 +266,7 @@ class BaseHandler:
 
         # If the response supports deferred rendering, apply template
         # response middleware and then render the response
-        if hasattr(response, 'render') and callable(response.render):
+        if hasattr(response, "render") and callable(response.render):
             for middleware_method in self._template_response_middleware:
                 response = await middleware_method(request, response)
                 # Complain if the template response middleware returned None or
@@ -252,15 +274,16 @@ class BaseHandler:
                 self.check_response(
                     response,
                     middleware_method,
-                    name='%s.process_template_response' % (
-                        middleware_method.__self__.__class__.__name__,
-                    )
+                    name="%s.process_template_response"
+                    % (middleware_method.__self__.__class__.__name__,),
                 )
             try:
                 if asyncio.iscoroutinefunction(response.render):
                     response = await response.render()
                 else:
-                    response = await sync_to_async(response.render, thread_sensitive=True)()
+                    response = await sync_to_async(
+                        response.render, thread_sensitive=True
+                    )()
             except Exception as e:
                 response = await sync_to_async(
                     self.process_exception_by_middleware,
@@ -271,7 +294,7 @@ class BaseHandler:
 
         # Make sure the response is not a coroutine
         if asyncio.iscoroutine(response):
-            raise RuntimeError('Response is still a coroutine.')
+            raise RuntimeError("Response is still a coroutine.")
         return response
 
     def resolve_request(self, request):
@@ -280,7 +303,7 @@ class BaseHandler:
         with its args and kwargs.
         """
         # Work out the resolver.
-        if hasattr(request, 'urlconf'):
+        if hasattr(request, "urlconf"):
             urlconf = request.urlconf
             set_urlconf(urlconf)
             resolver = get_resolver(urlconf)
@@ -295,13 +318,13 @@ class BaseHandler:
         """
         Raise an error if the view returned None or an uncalled coroutine.
         """
-        if not(response is None or asyncio.iscoroutine(response)):
+        if not (response is None or asyncio.iscoroutine(response)):
             return
         if not name:
             if isinstance(callback, types.FunctionType):  # FBV
-                name = 'The view %s.%s' % (callback.__module__, callback.__name__)
+                name = "The view %s.%s" % (callback.__module__, callback.__name__)
             else:  # CBV
-                name = 'The view %s.%s.__call__' % (
+                name = "The view %s.%s.__call__" % (
                     callback.__module__,
                     callback.__class__.__name__,
                 )
@@ -320,12 +343,15 @@ class BaseHandler:
     # Other utility methods.
 
     def make_view_atomic(self, view):
-        non_atomic_requests = getattr(view, '_non_atomic_requests', set())
+        non_atomic_requests = getattr(view, "_non_atomic_requests", set())
         for db in connections.all():
-            if db.settings_dict['ATOMIC_REQUESTS'] and db.alias not in non_atomic_requests:
+            if (
+                db.settings_dict["ATOMIC_REQUESTS"]
+                and db.alias not in non_atomic_requests
+            ):
                 if asyncio.iscoroutinefunction(view):
                     raise RuntimeError(
-                        'You cannot use ATOMIC_REQUESTS with async views.'
+                        "You cannot use ATOMIC_REQUESTS with async views."
                     )
                 view = transaction.atomic(using=db.alias)(view)
         return view
