@@ -2,8 +2,11 @@ import copy
 import datetime
 import re
 
-from django.db.backends.base.schema import BaseDatabaseSchemaEditor
-from django.db.utils import DatabaseError
+from django.db import DatabaseError
+from django.db.backends.base.schema import (
+    BaseDatabaseSchemaEditor,
+    _related_non_m2m_objects,
+)
 
 
 class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
@@ -14,8 +17,13 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
     sql_alter_column_not_null = "MODIFY %(column)s NOT NULL"
     sql_alter_column_default = "MODIFY %(column)s DEFAULT %(default)s"
     sql_alter_column_no_default = "MODIFY %(column)s DEFAULT NULL"
+    sql_alter_column_no_default_null = sql_alter_column_no_default
+    sql_alter_column_collate = "MODIFY %(column)s %(type)s%(collation)s"
+
     sql_delete_column = "ALTER TABLE %(table)s DROP COLUMN %(column)s"
-    sql_create_column_inline_fk = 'CONSTRAINT %(name)s REFERENCES %(to_table)s(%(to_column)s)%(deferrable)s'
+    sql_create_column_inline_fk = (
+        "CONSTRAINT %(name)s REFERENCES %(to_table)s(%(to_column)s)%(deferrable)s"
+    )
     sql_delete_table = "DROP TABLE %(table)s CASCADE CONSTRAINTS"
     sql_create_index = "CREATE INDEX %(name)s ON %(table)s (%(columns)s)%(extra)s"
 
@@ -23,7 +31,7 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
         if isinstance(value, (datetime.date, datetime.time, datetime.datetime)):
             return "'%s'" % value
         elif isinstance(value, str):
-            return "'%s'" % value.replace("\'", "\'\'").replace('%', '%%')
+            return "'%s'" % value.replace("'", "''").replace("%", "%%")
         elif isinstance(value, (bytes, bytearray, memoryview)):
             return "'%s'" % value.hex()
         elif isinstance(value, bool):
@@ -42,7 +50,8 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
         # Run superclass action
         super().delete_model(model)
         # Clean up manually created sequence.
-        self.execute("""
+        self.execute(
+            """
             DECLARE
                 i INTEGER;
             BEGIN
@@ -52,7 +61,13 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
                     EXECUTE IMMEDIATE 'DROP SEQUENCE "%(sq_name)s"';
                 END IF;
             END;
-        /""" % {'sq_name': self.connection.ops._get_no_autofield_sequence_name(model._meta.db_table)})
+        /"""
+            % {
+                "sq_name": self.connection.ops._get_no_autofield_sequence_name(
+                    model._meta.db_table
+                )
+            }
+        )
 
     def alter_field(self, model, old_field, new_field, strict=False):
         try:
@@ -61,16 +76,16 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
             description = str(e)
             # If we're changing type to an unsupported type we need a
             # SQLite-ish workaround
-            if 'ORA-22858' in description or 'ORA-22859' in description:
+            if "ORA-22858" in description or "ORA-22859" in description:
                 self._alter_field_type_workaround(model, old_field, new_field)
             # If an identity column is changing to a non-numeric type, drop the
             # identity first.
-            elif 'ORA-30675' in description:
+            elif "ORA-30675" in description:
                 self._drop_identity(model._meta.db_table, old_field.column)
                 self.alter_field(model, old_field, new_field, strict)
             # If a primary key column is changing to an identity column, drop
             # the primary key first.
-            elif 'ORA-30673' in description and old_field.primary_key:
+            elif "ORA-30673" in description and old_field.primary_key:
                 self._delete_primary_key(model, strict=True)
                 self._alter_field_type_workaround(model, old_field, new_field)
             else:
@@ -90,37 +105,69 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
         # Make a new field that's like the new one but with a temporary
         # column name.
         new_temp_field = copy.deepcopy(new_field)
-        new_temp_field.null = (new_field.get_internal_type() not in ('AutoField', 'BigAutoField', 'SmallAutoField'))
+        new_temp_field.null = new_field.get_internal_type() not in (
+            "AutoField",
+            "BigAutoField",
+            "SmallAutoField",
+        )
         new_temp_field.column = self._generate_temp_name(new_field.column)
         # Add it
         self.add_field(model, new_temp_field)
         # Explicit data type conversion
-        # https://docs.oracle.com/en/database/oracle/oracle-database/18/sqlrf
+        # https://docs.oracle.com/en/database/oracle/oracle-database/21/sqlrf
         # /Data-Type-Comparison-Rules.html#GUID-D0C5A47E-6F93-4C2D-9E49-4F2B86B359DD
         new_value = self.quote_name(old_field.column)
         old_type = old_field.db_type(self.connection)
-        if re.match('^N?CLOB', old_type):
+        if re.match("^N?CLOB", old_type):
             new_value = "TO_CHAR(%s)" % new_value
-            old_type = 'VARCHAR2'
-        if re.match('^N?VARCHAR2', old_type):
+            old_type = "VARCHAR2"
+        if re.match("^N?VARCHAR2", old_type):
             new_internal_type = new_field.get_internal_type()
-            if new_internal_type == 'DateField':
+            if new_internal_type == "DateField":
                 new_value = "TO_DATE(%s, 'YYYY-MM-DD')" % new_value
-            elif new_internal_type == 'DateTimeField':
+            elif new_internal_type == "DateTimeField":
                 new_value = "TO_TIMESTAMP(%s, 'YYYY-MM-DD HH24:MI:SS.FF')" % new_value
-            elif new_internal_type == 'TimeField':
+            elif new_internal_type == "TimeField":
                 # TimeField are stored as TIMESTAMP with a 1900-01-01 date part.
-                new_value = "TO_TIMESTAMP(CONCAT('1900-01-01 ', %s), 'YYYY-MM-DD HH24:MI:SS.FF')" % new_value
+                new_value = "CONCAT('1900-01-01 ', %s)" % new_value
+                new_value = "TO_TIMESTAMP(%s, 'YYYY-MM-DD HH24:MI:SS.FF')" % new_value
         # Transfer values across
-        self.execute("UPDATE %s set %s=%s" % (
-            self.quote_name(model._meta.db_table),
-            self.quote_name(new_temp_field.column),
-            new_value,
-        ))
+        self.execute(
+            "UPDATE %s set %s=%s"
+            % (
+                self.quote_name(model._meta.db_table),
+                self.quote_name(new_temp_field.column),
+                new_value,
+            )
+        )
         # Drop the old field
         self.remove_field(model, old_field)
         # Rename and possibly make the new field NOT NULL
         super().alter_field(model, new_temp_field, new_field)
+        # Recreate foreign key (if necessary) because the old field is not
+        # passed to the alter_field() and data types of new_temp_field and
+        # new_field always match.
+        new_type = new_field.db_type(self.connection)
+        if (
+            (old_field.primary_key and new_field.primary_key)
+            or (old_field.unique and new_field.unique)
+        ) and old_type != new_type:
+            for _, rel in _related_non_m2m_objects(new_temp_field, new_field):
+                if rel.field.db_constraint:
+                    self.execute(
+                        self._create_fk_sql(rel.related_model, rel.field, "_fk")
+                    )
+
+    def _alter_column_type_sql(self, model, old_field, new_field, new_type):
+        auto_field_types = {"AutoField", "BigAutoField", "SmallAutoField"}
+        # Drop the identity if migrating away from AutoField.
+        if (
+            old_field.get_internal_type() in auto_field_types
+            and new_field.get_internal_type() not in auto_field_types
+            and self._is_identity_column(model._meta.db_table, new_field.column)
+        ):
+            self._drop_identity(model._meta.db_table, new_field.column)
+        return super()._alter_column_type_sql(model, old_field, new_field, new_type)
 
     def normalize_name(self, name):
         """
@@ -143,30 +190,55 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
     def _field_should_be_indexed(self, model, field):
         create_index = super()._field_should_be_indexed(model, field)
         db_type = field.db_type(self.connection)
-        if db_type is not None and db_type.lower() in self.connection._limited_data_types:
+        if (
+            db_type is not None
+            and db_type.lower() in self.connection._limited_data_types
+        ):
             return False
         return create_index
 
     def _unique_should_be_added(self, old_field, new_field):
-        return (
-            super()._unique_should_be_added(old_field, new_field) and
-            not self._field_became_primary_key(old_field, new_field)
-        )
+        return super()._unique_should_be_added(
+            old_field, new_field
+        ) and not self._field_became_primary_key(old_field, new_field)
 
     def _is_identity_column(self, table_name, column_name):
         with self.connection.cursor() as cursor:
-            cursor.execute("""
+            cursor.execute(
+                """
                 SELECT
                     CASE WHEN identity_column = 'YES' THEN 1 ELSE 0 END
                 FROM user_tab_cols
                 WHERE table_name = %s AND
                       column_name = %s
-            """, [self.normalize_name(table_name), self.normalize_name(column_name)])
+                """,
+                [self.normalize_name(table_name), self.normalize_name(column_name)],
+            )
             row = cursor.fetchone()
             return row[0] if row else False
 
     def _drop_identity(self, table_name, column_name):
-        self.execute('ALTER TABLE %(table)s MODIFY %(column)s DROP IDENTITY' % {
-            'table': self.quote_name(table_name),
-            'column': self.quote_name(column_name),
-        })
+        self.execute(
+            "ALTER TABLE %(table)s MODIFY %(column)s DROP IDENTITY"
+            % {
+                "table": self.quote_name(table_name),
+                "column": self.quote_name(column_name),
+            }
+        )
+
+    def _get_default_collation(self, table_name):
+        with self.connection.cursor() as cursor:
+            cursor.execute(
+                """
+                SELECT default_collation FROM user_tables WHERE table_name = %s
+                """,
+                [self.normalize_name(table_name)],
+            )
+            return cursor.fetchone()[0]
+
+    def _alter_column_collation_sql(self, model, new_field, new_type, new_collation):
+        if new_collation is None:
+            new_collation = self._get_default_collation(model._meta.db_table)
+        return super()._alter_column_collation_sql(
+            model, new_field, new_type, new_collation
+        )
