@@ -6,23 +6,24 @@ from itertools import chain
 
 from django.conf import settings
 from django.core.exceptions import FieldError
-from django.db import utils
+from django.db import DatabaseError, NotSupportedError, models
 from django.db.backends.base.operations import BaseDatabaseOperations
-from django.db.models import aggregates, fields
 from django.db.models.expressions import Col
 from django.utils import timezone
 from django.utils.dateparse import parse_date, parse_datetime, parse_time
-from django.utils.duration import duration_microseconds
 from django.utils.functional import cached_property
 
 
 class DatabaseOperations(BaseDatabaseOperations):
-    cast_char_field_without_max_length = 'text'
+    cast_char_field_without_max_length = "text"
     cast_data_types = {
-        'DateField': 'TEXT',
-        'DateTimeField': 'TEXT',
+        "DateField": "TEXT",
+        "DateTimeField": "TEXT",
     }
-    explain_prefix = 'EXPLAIN QUERY PLAN'
+    explain_prefix = "EXPLAIN QUERY PLAN"
+    # List of datatypes to that cannot be extracted with JSON_EXTRACT() on
+    # SQLite. Use JSON_TYPE() instead.
+    jsonfield_datatype_values = frozenset(["null", "false", "true"])
 
     def bulk_batch_size(self, fields, objs):
         """
@@ -40,25 +41,29 @@ class DatabaseOperations(BaseDatabaseOperations):
             return len(objs)
 
     def check_expression_support(self, expression):
-        bad_fields = (fields.DateField, fields.DateTimeField, fields.TimeField)
-        bad_aggregates = (aggregates.Sum, aggregates.Avg, aggregates.Variance, aggregates.StdDev)
+        bad_fields = (models.DateField, models.DateTimeField, models.TimeField)
+        bad_aggregates = (models.Sum, models.Avg, models.Variance, models.StdDev)
         if isinstance(expression, bad_aggregates):
             for expr in expression.get_source_expressions():
                 try:
                     output_field = expr.output_field
-                except FieldError:
+                except (AttributeError, FieldError):
                     # Not every subexpression has an output_field which is fine
                     # to ignore.
                     pass
                 else:
                     if isinstance(output_field, bad_fields):
-                        raise utils.NotSupportedError(
-                            'You cannot use Sum, Avg, StdDev, and Variance '
-                            'aggregations on date/time fields in sqlite3 '
-                            'since date/time is saved as text.'
+                        raise NotSupportedError(
+                            "You cannot use Sum, Avg, StdDev, and Variance "
+                            "aggregations on date/time fields in sqlite3 "
+                            "since date/time is saved as text."
                         )
-        if isinstance(expression, aggregates.Aggregate) and len(expression.source_expressions) > 1:
-            raise utils.NotSupportedError(
+        if (
+            isinstance(expression, models.Aggregate)
+            and expression.distinct
+            and len(expression.source_expressions) > 1
+        ):
+            raise NotSupportedError(
                 "SQLite doesn't support DISTINCT on aggregate functions "
                 "accepting multiple arguments."
             )
@@ -71,42 +76,60 @@ class DatabaseOperations(BaseDatabaseOperations):
         """
         return "django_date_extract('%s', %s)" % (lookup_type.lower(), field_name)
 
-    def date_interval_sql(self, timedelta):
-        return str(duration_microseconds(timedelta))
+    def fetch_returned_insert_rows(self, cursor):
+        """
+        Given a cursor object that has just performed an INSERT...RETURNING
+        statement into a table, return the list of returned data.
+        """
+        return cursor.fetchall()
 
     def format_for_duration_arithmetic(self, sql):
         """Do nothing since formatting is handled in the custom function."""
         return sql
 
-    def date_trunc_sql(self, lookup_type, field_name):
-        return "django_date_trunc('%s', %s)" % (lookup_type.lower(), field_name)
+    def date_trunc_sql(self, lookup_type, field_name, tzname=None):
+        return "django_date_trunc('%s', %s, %s, %s)" % (
+            lookup_type.lower(),
+            field_name,
+            *self._convert_tznames_to_sql(tzname),
+        )
 
-    def time_trunc_sql(self, lookup_type, field_name):
-        return "django_time_trunc('%s', %s)" % (lookup_type.lower(), field_name)
+    def time_trunc_sql(self, lookup_type, field_name, tzname=None):
+        return "django_time_trunc('%s', %s, %s, %s)" % (
+            lookup_type.lower(),
+            field_name,
+            *self._convert_tznames_to_sql(tzname),
+        )
 
     def _convert_tznames_to_sql(self, tzname):
-        if settings.USE_TZ:
+        if tzname and settings.USE_TZ:
             return "'%s'" % tzname, "'%s'" % self.connection.timezone_name
-        return 'NULL', 'NULL'
+        return "NULL", "NULL"
 
     def datetime_cast_date_sql(self, field_name, tzname):
-        return 'django_datetime_cast_date(%s, %s, %s)' % (
-            field_name, *self._convert_tznames_to_sql(tzname),
+        return "django_datetime_cast_date(%s, %s, %s)" % (
+            field_name,
+            *self._convert_tznames_to_sql(tzname),
         )
 
     def datetime_cast_time_sql(self, field_name, tzname):
-        return 'django_datetime_cast_time(%s, %s, %s)' % (
-            field_name, *self._convert_tznames_to_sql(tzname),
+        return "django_datetime_cast_time(%s, %s, %s)" % (
+            field_name,
+            *self._convert_tznames_to_sql(tzname),
         )
 
     def datetime_extract_sql(self, lookup_type, field_name, tzname):
         return "django_datetime_extract('%s', %s, %s, %s)" % (
-            lookup_type.lower(), field_name, *self._convert_tznames_to_sql(tzname),
+            lookup_type.lower(),
+            field_name,
+            *self._convert_tznames_to_sql(tzname),
         )
 
     def datetime_trunc_sql(self, lookup_type, field_name, tzname):
         return "django_datetime_trunc('%s', %s, %s, %s)" % (
-            lookup_type.lower(), field_name, *self._convert_tznames_to_sql(tzname),
+            lookup_type.lower(),
+            field_name,
+            *self._convert_tznames_to_sql(tzname),
         )
 
     def time_extract_sql(self, lookup_type, field_name):
@@ -128,11 +151,11 @@ class DatabaseOperations(BaseDatabaseOperations):
         if len(params) > BATCH_SIZE:
             results = ()
             for index in range(0, len(params), BATCH_SIZE):
-                chunk = params[index:index + BATCH_SIZE]
+                chunk = params[index : index + BATCH_SIZE]
                 results += self._quote_params_for_last_executed_query(chunk)
             return results
 
-        sql = 'SELECT ' + ', '.join(['QUOTE(?)'] * len(params))
+        sql = "SELECT " + ", ".join(["QUOTE(?)"] * len(params))
         # Bypass Django's wrappers and use the underlying sqlite3 connection
         # to avoid logging this query - it would trigger infinite recursion.
         cursor = self.connection.connection.cursor()
@@ -144,7 +167,9 @@ class DatabaseOperations(BaseDatabaseOperations):
 
     def last_executed_query(self, cursor, sql, params):
         # Python substitutes parameters in Modules/_sqlite/cursor.c with:
-        # pysqlite_statement_bind_parameters(self->statement, parameters, allow_8bit_chars);
+        # pysqlite_statement_bind_parameters(
+        #   self->statement, parameters, allow_8bit_chars
+        # );
         # Unfortunately there is no way to reach self->statement from Python,
         # so we quote and substitute parameters manually.
         if params:
@@ -193,25 +218,52 @@ class DatabaseOperations(BaseDatabaseOperations):
         # Django's test suite.
         return lru_cache(maxsize=512)(self.__references_graph)
 
-    def sql_flush(self, style, tables, sequences, allow_cascade=False):
+    def sql_flush(self, style, tables, *, reset_sequences=False, allow_cascade=False):
         if tables and allow_cascade:
             # Simulate TRUNCATE CASCADE by recursively collecting the tables
             # referencing the tables to be flushed.
-            tables = set(chain.from_iterable(self._references_graph(table) for table in tables))
-        # Note: No requirement for reset of auto-incremented indices (cf. other
-        # sql_flush() implementations). Just return SQL at this point
-        return ['%s %s %s;' % (
-            style.SQL_KEYWORD('DELETE'),
-            style.SQL_KEYWORD('FROM'),
-            style.SQL_FIELD(self.quote_name(table))
-        ) for table in tables]
+            tables = set(
+                chain.from_iterable(self._references_graph(table) for table in tables)
+            )
+        sql = [
+            "%s %s %s;"
+            % (
+                style.SQL_KEYWORD("DELETE"),
+                style.SQL_KEYWORD("FROM"),
+                style.SQL_FIELD(self.quote_name(table)),
+            )
+            for table in tables
+        ]
+        if reset_sequences:
+            sequences = [{"table": table} for table in tables]
+            sql.extend(self.sequence_reset_by_name_sql(style, sequences))
+        return sql
+
+    def sequence_reset_by_name_sql(self, style, sequences):
+        if not sequences:
+            return []
+        return [
+            "%s %s %s %s = 0 %s %s %s (%s);"
+            % (
+                style.SQL_KEYWORD("UPDATE"),
+                style.SQL_TABLE(self.quote_name("sqlite_sequence")),
+                style.SQL_KEYWORD("SET"),
+                style.SQL_FIELD(self.quote_name("seq")),
+                style.SQL_KEYWORD("WHERE"),
+                style.SQL_FIELD(self.quote_name("name")),
+                style.SQL_KEYWORD("IN"),
+                ", ".join(
+                    ["'%s'" % sequence_info["table"] for sequence_info in sequences]
+                ),
+            ),
+        ]
 
     def adapt_datetimefield_value(self, value):
         if value is None:
             return None
 
         # Expression values are adapted by the database.
-        if hasattr(value, 'resolve_expression'):
+        if hasattr(value, "resolve_expression"):
             return value
 
         # SQLite doesn't support tz-aware datetimes
@@ -219,7 +271,10 @@ class DatabaseOperations(BaseDatabaseOperations):
             if settings.USE_TZ:
                 value = timezone.make_naive(value, self.connection.timezone)
             else:
-                raise ValueError("SQLite backend does not support timezone-aware datetimes when USE_TZ is False.")
+                raise ValueError(
+                    "SQLite backend does not support timezone-aware datetimes when "
+                    "USE_TZ is False."
+                )
 
         return str(value)
 
@@ -228,7 +283,7 @@ class DatabaseOperations(BaseDatabaseOperations):
             return None
 
         # Expression values are adapted by the database.
-        if hasattr(value, 'resolve_expression'):
+        if hasattr(value, "resolve_expression"):
             return value
 
         # SQLite doesn't support tz-aware datetimes
@@ -240,17 +295,17 @@ class DatabaseOperations(BaseDatabaseOperations):
     def get_db_converters(self, expression):
         converters = super().get_db_converters(expression)
         internal_type = expression.output_field.get_internal_type()
-        if internal_type == 'DateTimeField':
+        if internal_type == "DateTimeField":
             converters.append(self.convert_datetimefield_value)
-        elif internal_type == 'DateField':
+        elif internal_type == "DateField":
             converters.append(self.convert_datefield_value)
-        elif internal_type == 'TimeField':
+        elif internal_type == "TimeField":
             converters.append(self.convert_timefield_value)
-        elif internal_type == 'DecimalField':
+        elif internal_type == "DecimalField":
             converters.append(self.get_decimalfield_converter(expression))
-        elif internal_type == 'UUIDField':
+        elif internal_type == "UUIDField":
             converters.append(self.convert_uuidfield_value)
-        elif internal_type in ('NullBooleanField', 'BooleanField'):
+        elif internal_type == "BooleanField":
             converters.append(self.convert_booleanfield_value)
         return converters
 
@@ -279,15 +334,22 @@ class DatabaseOperations(BaseDatabaseOperations):
         # float inaccuracy must be removed.
         create_decimal = decimal.Context(prec=15).create_decimal_from_float
         if isinstance(expression, Col):
-            quantize_value = decimal.Decimal(1).scaleb(-expression.output_field.decimal_places)
+            quantize_value = decimal.Decimal(1).scaleb(
+                -expression.output_field.decimal_places
+            )
 
             def converter(value, expression, connection):
                 if value is not None:
-                    return create_decimal(value).quantize(quantize_value, context=expression.output_field.context)
+                    return create_decimal(value).quantize(
+                        quantize_value, context=expression.output_field.context
+                    )
+
         else:
+
             def converter(value, expression, connection):
                 if value is not None:
                     return create_decimal(value)
+
         return converter
 
     def convert_uuidfield_value(self, value, expression, connection):
@@ -300,24 +362,25 @@ class DatabaseOperations(BaseDatabaseOperations):
 
     def bulk_insert_sql(self, fields, placeholder_rows):
         return " UNION ALL ".join(
-            "SELECT %s" % ", ".join(row)
-            for row in placeholder_rows
+            "SELECT %s" % ", ".join(row) for row in placeholder_rows
         )
 
     def combine_expression(self, connector, sub_expressions):
         # SQLite doesn't have a ^ operator, so use the user-defined POWER
         # function that's registered in connect().
-        if connector == '^':
-            return 'POWER(%s)' % ','.join(sub_expressions)
+        if connector == "^":
+            return "POWER(%s)" % ",".join(sub_expressions)
+        elif connector == "#":
+            return "BITXOR(%s)" % ",".join(sub_expressions)
         return super().combine_expression(connector, sub_expressions)
 
     def combine_duration_expression(self, connector, sub_expressions):
-        if connector not in ['+', '-']:
-            raise utils.DatabaseError('Invalid connector for timedelta: %s.' % connector)
+        if connector not in ["+", "-", "*", "/"]:
+            raise DatabaseError("Invalid connector for timedelta: %s." % connector)
         fn_params = ["'%s'" % connector] + sub_expressions
         if len(fn_params) > 3:
-            raise ValueError('Too many params for timedelta operations.')
-        return "django_format_dtdelta(%s)" % ', '.join(fn_params)
+            raise ValueError("Too many params for timedelta operations.")
+        return "django_format_dtdelta(%s)" % ", ".join(fn_params)
 
     def integer_field_range(self, internal_type):
         # SQLite doesn't enforce any integer constraints
@@ -327,9 +390,27 @@ class DatabaseOperations(BaseDatabaseOperations):
         lhs_sql, lhs_params = lhs
         rhs_sql, rhs_params = rhs
         params = (*lhs_params, *rhs_params)
-        if internal_type == 'TimeField':
-            return 'django_time_diff(%s, %s)' % (lhs_sql, rhs_sql), params
-        return 'django_timestamp_diff(%s, %s)' % (lhs_sql, rhs_sql), params
+        if internal_type == "TimeField":
+            return "django_time_diff(%s, %s)" % (lhs_sql, rhs_sql), params
+        return "django_timestamp_diff(%s, %s)" % (lhs_sql, rhs_sql), params
 
     def insert_statement(self, ignore_conflicts=False):
-        return 'INSERT OR IGNORE INTO' if ignore_conflicts else super().insert_statement(ignore_conflicts)
+        return (
+            "INSERT OR IGNORE INTO"
+            if ignore_conflicts
+            else super().insert_statement(ignore_conflicts)
+        )
+
+    def return_insert_columns(self, fields):
+        # SQLite < 3.35 doesn't support an INSERT...RETURNING statement.
+        if not fields:
+            return "", ()
+        columns = [
+            "%s.%s"
+            % (
+                self.quote_name(field.model._meta.db_table),
+                self.quote_name(field.column),
+            )
+            for field in fields
+        ]
+        return "RETURNING %s" % ", ".join(columns), ()
