@@ -1,6 +1,6 @@
-from django.core.exceptions import EmptyResultSet
 from django.db.models.sql.query import *  # NOQA
+from django.db.models.sql.query import Query
 from django.db.models.sql.subqueries import *  # NOQA
 from django.db.models.sql.where import AND, OR
 
-__all__ = ['Query', 'AND', 'OR', 'EmptyResultSet']
+__all__ = ["Query", "AND", "OR"]
