@@ -1,16 +1,18 @@
 import logging
-from functools import update_wrapper
 
 from django.core.exceptions import ImproperlyConfigured
 from django.http import (
-    HttpResponse, HttpResponseGone, HttpResponseNotAllowed,
-    HttpResponsePermanentRedirect, HttpResponseRedirect,
+    HttpResponse,
+    HttpResponseGone,
+    HttpResponseNotAllowed,
+    HttpResponsePermanentRedirect,
+    HttpResponseRedirect,
 )
 from django.template.response import TemplateResponse
 from django.urls import reverse
 from django.utils.decorators import classonlymethod
 
-logger = logging.getLogger('django.request')
+logger = logging.getLogger("django.request")
 
 
 class ContextMixin:
@@ -18,10 +20,11 @@ class ContextMixin:
     A default context mixin that passes the keyword arguments received by
     get_context_data() as the template context.
     """
+
     extra_context = None
 
     def get_context_data(self, **kwargs):
-        kwargs.setdefault('view', self)
+        kwargs.setdefault("view", self)
         if self.extra_context is not None:
             kwargs.update(self.extra_context)
         return kwargs
@@ -33,7 +36,16 @@ class View:
     dispatch-by-method and simple sanity checking.
     """
 
-    http_method_names = ['get', 'post', 'put', 'patch', 'delete', 'head', 'options', 'trace']
+    http_method_names = [
+        "get",
+        "post",
+        "put",
+        "patch",
+        "delete",
+        "head",
+        "options",
+        "trace",
+    ]
 
     def __init__(self, **kwargs):
         """
@@ -51,37 +63,44 @@ class View:
         for key in initkwargs:
             if key in cls.http_method_names:
                 raise TypeError(
-                    'The method name %s is not accepted as a keyword argument '
-                    'to %s().' % (key, cls.__name__)
+                    "The method name %s is not accepted as a keyword argument "
+                    "to %s()." % (key, cls.__name__)
                 )
             if not hasattr(cls, key):
-                raise TypeError("%s() received an invalid keyword %r. as_view "
-                                "only accepts arguments that are already "
-                                "attributes of the class." % (cls.__name__, key))
+                raise TypeError(
+                    "%s() received an invalid keyword %r. as_view "
+                    "only accepts arguments that are already "
+                    "attributes of the class." % (cls.__name__, key)
+                )
 
         def view(request, *args, **kwargs):
             self = cls(**initkwargs)
             self.setup(request, *args, **kwargs)
-            if not hasattr(self, 'request'):
+            if not hasattr(self, "request"):
                 raise AttributeError(
                     "%s instance has no 'request' attribute. Did you override "
                     "setup() and forget to call super()?" % cls.__name__
                 )
             return self.dispatch(request, *args, **kwargs)
+
         view.view_class = cls
         view.view_initkwargs = initkwargs
 
-        # take name and docstring from class
-        update_wrapper(view, cls, updated=())
+        # __name__ and __qualname__ are intentionally left unchanged as
+        # view_class should be used to robustly determine the name of the view
+        # instead.
+        view.__doc__ = cls.__doc__
+        view.__module__ = cls.__module__
+        view.__annotations__ = cls.dispatch.__annotations__
+        # Copy possible attributes set by decorators, e.g. @csrf_exempt, from
+        # the dispatch method.
+        view.__dict__.update(cls.dispatch.__dict__)
 
-        # and possible attributes set by decorators
-        # like csrf_exempt from dispatch
-        update_wrapper(view, cls.dispatch, assigned=())
         return view
 
     def setup(self, request, *args, **kwargs):
         """Initialize attributes shared by all view methods."""
-        if hasattr(self, 'get') and not hasattr(self, 'head'):
+        if hasattr(self, "get") and not hasattr(self, "head"):
             self.head = self.get
         self.request = request
         self.args = args
@@ -92,23 +111,27 @@ class View:
         # defer to the error handler. Also defer to the error handler if the
         # request method isn't on the approved list.
         if request.method.lower() in self.http_method_names:
-            handler = getattr(self, request.method.lower(), self.http_method_not_allowed)
+            handler = getattr(
+                self, request.method.lower(), self.http_method_not_allowed
+            )
         else:
             handler = self.http_method_not_allowed
         return handler(request, *args, **kwargs)
 
     def http_method_not_allowed(self, request, *args, **kwargs):
         logger.warning(
-            'Method Not Allowed (%s): %s', request.method, request.path,
-            extra={'status_code': 405, 'request': request}
+            "Method Not Allowed (%s): %s",
+            request.method,
+            request.path,
+            extra={"status_code": 405, "request": request},
         )
         return HttpResponseNotAllowed(self._allowed_methods())
 
     def options(self, request, *args, **kwargs):
         """Handle responding to requests for the OPTIONS HTTP verb."""
         response = HttpResponse()
-        response['Allow'] = ', '.join(self._allowed_methods())
-        response['Content-Length'] = '0'
+        response.headers["Allow"] = ", ".join(self._allowed_methods())
+        response.headers["Content-Length"] = "0"
         return response
 
     def _allowed_methods(self):
@@ -117,6 +140,7 @@ class View:
 
 class TemplateResponseMixin:
     """A mixin that can be used to render a template."""
+
     template_name = None
     template_engine = None
     response_class = TemplateResponse
@@ -129,13 +153,13 @@ class TemplateResponseMixin:
 
         Pass response_kwargs to the constructor of the response class.
         """
-        response_kwargs.setdefault('content_type', self.content_type)
+        response_kwargs.setdefault("content_type", self.content_type)
         return self.response_class(
             request=self.request,
             template=self.get_template_names(),
             context=context,
             using=self.template_engine,
-            **response_kwargs
+            **response_kwargs,
         )
 
     def get_template_names(self):
@@ -146,7 +170,8 @@ class TemplateResponseMixin:
         if self.template_name is None:
             raise ImproperlyConfigured(
                 "TemplateResponseMixin requires either a definition of "
-                "'template_name' or an implementation of 'get_template_names()'")
+                "'template_name' or an implementation of 'get_template_names()'"
+            )
         else:
             return [self.template_name]
 
@@ -155,6 +180,7 @@ class TemplateView(TemplateResponseMixin, ContextMixin, View):
     """
     Render a template. Pass keyword arguments from the URLconf to the context.
     """
+
     def get(self, request, *args, **kwargs):
         context = self.get_context_data(**kwargs)
         return self.render_to_response(context)
@@ -162,6 +188,7 @@ class TemplateView(TemplateResponseMixin, ContextMixin, View):
 
 class RedirectView(View):
     """Provide a redirect on any GET request."""
+
     permanent = False
     url = None
     pattern_name = None
@@ -180,7 +207,7 @@ class RedirectView(View):
         else:
             return None
 
-        args = self.request.META.get('QUERY_STRING', '')
+        args = self.request.META.get("QUERY_STRING", "")
         if args and self.query_string:
             url = "%s?%s" % (url, args)
         return url
@@ -194,8 +221,7 @@ class RedirectView(View):
                 return HttpResponseRedirect(url)
         else:
             logger.warning(
-                'Gone: %s', request.path,
-                extra={'status_code': 410, 'request': request}
+                "Gone: %s", request.path, extra={"status_code": 410, "request": request}
             )
             return HttpResponseGone()
 
